import config from './config.mjs';
import {deleteCookie, getCookie} from './tools.mjs';

const defaultPosition = { 
	"lat": 47.498375, "lng": 19.040529 
};

const worldBorders = [
	{lat: -85.1054596961173, lng: -180},
	{lat: 85.1054596961173, lng: -180},
	{lat: 85.1054596961173, lng: 180},
	{lat: -85.1054596961173, lng: 180},
	{lat: -85.1054596961173, lng: 0}
];

const pinSymbolColors = [
	"#DF0101","#DF7401","#DBA901","#D7DF01","#A5DF00","#01DF01","#FFFFFF"	
];
const pinSymbolStrokeColors = [
	"#610B0B","#61380B","#5F4C0B","#5E610B","#4B610B","#21610B","#000000"
];

const adrMargin = 10.0;

const adrTresholds = [
	/*-19.0,*/-16.5,-14.0,-11.5,-9.0,-7.0
];

let pinSymbolScale = 5;

let map = null;
let currentPosition = defaultPosition;
let bounds = null;
let gateways = null;

let markers = [];
let gw_markers = [];
let infoWindow = null;
let markerCluster = null;

let filter = {};
let queryAreaLines = null;
let numberOfPages = 0;

let points = {};

// let access_token = localStorage.getItem( localStoragePrefix + 'token' );
let access_token = getCookie( 'access_token_' + config.client_id );

let FCntUp = 0;
let pos = {};
let DevEUI = "";

function pinSymbol(colorIndex,scale) {
		return {
			//path: 'M 0,0 C -2,-20 -10,-22 -10,-30 A 10,10 0 1,1 10,-30 C 10,-22 2,-20 0,0 z',
			path: google.maps.SymbolPath.CIRCLE,
			fillColor: pinSymbolColors[colorIndex],
			fillOpacity: 1,
			strokeColor: pinSymbolStrokeColors[colorIndex],
			strokeWeight: 1,
			scale: parseFloat($('#marker_size').val()) //pinSymbolScale
	 };
}

function get_gw_points(done) {
}

function set_gw_markers() {

	if ( gw_markers.length > 0 ) {
		hide_gw_markers();
		gw_markers = [];
	}

	$.each( gateways, function(key, gateway){

		if (gateway.position!=null) {
		
			let marker = new google.maps.Marker({
				position: gateway.position, 
				title: gateway.gatewayId+' | '+gateway.name,
				optimized: false,
				zIndex: 10,
				icon: {
					url: "assets/images/gw.png",
					size: new google.maps.Size(50, 50),
					origin: new google.maps.Point(0, 0),
					anchor: new google.maps.Point(18, 18),
					scaledSize: new google.maps.Size(36, 36)
				}
			});

			google.maps.event.addListener(marker, 'click', function() { 
				infoWindow.setOptions({
					content: '<pre>'+JSON.stringify(gateway, null, 2)+'</pre>',
					maxWidth: 400
				});
				infoWindow.open(map, this); 
			});

			gw_markers.push(marker);
		
		}
		
	});


	show_gw_markers()	

}

function show_gw_markers() {
	hide_gw_markers();			
	gw_markers.forEach(function(marker, i){
		marker.setMap(map);
	});
}

function hide_gw_markers() {
	gw_markers.forEach(function(marker, i){
		marker.setMap(null);
	});
}


function set_filter(done) {

	filter = {};

	if ($('#DevEUI').val()!="") filter.DevEUI = $('#DevEUI').val();
	if ($('#DevAddr').val()!="") filter.DevAddr = $('#DevAddr').val();
	if ($('#SpFact').val()!="") filter.SpFact = $('#SpFact').val();

	filter.ADRbit = $('input[name=ADRbit]:checked').val();

	if ($('#bestLrrid').val()!="") filter.bestLrrid = $('#bestLrrid').val();
	if ($('#Lrrid').val()!="") filter.Lrrid = $('#Lrrid').val();
	if ($('#startTime').val()!="") filter.startTime = $('#startTime').val();
	if ($('#endTime').val()!="") filter.endTime = $('#endTime').val();

	if( ($('#DevEUI').val()=="") && ($('#DevAddr').val()=="") ) {
		$('.ui.checkbox.showLostMessages').checkbox('uncheck');
	}
	if( ($('#SpFact').val()!="") ) {
		$('.ui.checkbox.showLostMessages').checkbox('uncheck');
	}

	let currentMapBounds = map.getBounds();

	if ($('.ui.radio.checkbox.qaBounds.manual').first().checkbox('is checked')) {

		filter.startLat = parseFloat($('#startLat').val());
		filter.startLng = parseFloat($('#startLng').val());
		filter.endLat = parseFloat($('#endLat').val());
		filter.endLng = parseFloat($('#endLng').val());

		if (isNaN(filter.startLat)) {
			filter.startLat = currentMapBounds.getSouthWest().lat();
			$('#startLat').val(filter.startLat);
			$('#startLat').parent().find('i').show();
		}
		if (isNaN(filter.startLng)) {
			filter.startLng = currentMapBounds.getSouthWest().lng();
			$('#startLng').val(filter.startLng);
			$('#startLng').parent().find('i').show();
		}
		if (isNaN(filter.endLat)) {
			filter.endLat = currentMapBounds.getNorthEast().lat();
			$('#endLat').val(filter.endLat);
			$('#endLat').parent().find('i').show();
		}
		if (isNaN(filter.endLng)) {
			filter.endLng = currentMapBounds.getNorthEast().lng();
			$('#endLng').val(filter.endLng);
			$('#endLng').parent().find('i').show();
		}
	}
	else {
		let gutter = Math.abs(currentMapBounds.getNorthEast().lng() - currentMapBounds.getSouthWest().lng()) / 100;
		filter.startLat = currentMapBounds.getSouthWest().lat()+gutter; 
		filter.startLng = currentMapBounds.getSouthWest().lng()+gutter; 
		filter.endLat = currentMapBounds.getNorthEast().lat()-gutter; 
		filter.endLng = currentMapBounds.getNorthEast().lng()-gutter;
	}

	if (done) done();

}


function get_points() {

	let data = filter;

	if ( parseInt($('#pageNumber').val()) > 0 ) data.pageNumber = $('#pageNumber').val();
	if ( parseInt($('#pageLength').val()) > 0 ) data.pageLength = $('#pageLength').val();

	$('body').addClass('waiting');

	$.ajax({
		type: "GET",
		url: config.apiURL + '/uplinkMsg',
		dataType: "json",
		headers: {"Authorization": 'Bearer ' + access_token},
		data: data
	})
	.done(function( response, textStatus, jqXHR ) {
		points = response.message;
		set_markers();
		set_query_area_lines();
		numberOfPages = Math.ceil(parseInt(response.numberOfMessages)/parseInt(data.pageLength));
		$("#numberOfPages").val( numberOfPages );
	})
	.fail(function( jqXHR, textStatus, errorThrown ) {
		if (jqXHR.status = 403) {
			// alert(jqXHR.responseJSON.message);
			location.href = 'login.html';
		} else {
			console.log('An error occurred.');
			console.log(errorThrown);
		}
		$('body').removeClass('waiting');
	})
}

function set_markers(done) {
	if ( markers.length > 0 ) {
		hide_markers();
		markers = [];
		// infowindows = [];
	}

	bounds = new google.maps.LatLngBounds();

	pos = null;

	$.each( points, function(key, point){

		if (point.DevEUI_uplink.payload_json.position!=null) {

			let posNew = point.DevEUI_uplink.payload_json.position;
			let FCntUpNew = parseInt(point.DevEUI_uplink.FCntUp);
			let DevEUINew = point.DevEUI_uplink.DevEUI;

			if (DevEUINew != DevEUI) {
				pos = null;
				FCntUp = 0;
				DevEUI = DevEUINew;
			}

			let numOfLostMessages = FCntUpNew - FCntUp - 1;

			if (numOfLostMessages < 0) {
				numOfLostMessages = FCntUpNew-1;
				pos = null;
				FCntUp = 0;
				DevEUI = DevEUINew;
			}

			if (
				$('#showLostMessages').prop('checked') &&
				pos &&
				(numOfLostMessages > 0) &&
				(numOfLostMessages < 100)
			) {

				// if (numOfLostMessages>100) numOfLostMessages = 100;

				let latVector = (parseFloat(posNew.lat)-pos.lat)/(numOfLostMessages+1); 
				let lngVector = (parseFloat(posNew.lng)-pos.lng)/(numOfLostMessages+1);

				for (let i=0; i<numOfLostMessages; i++) {

					pos.lat += latVector;
					pos.lng += lngVector;
					FCntUp++;

					let marker = new google.maps.Marker({
						position: pos, 
						title: FCntUp + ' | Lost | ' + DevEUI,
						optimized: false,
						zIndex: 5,
						icon: pinSymbol(6)
					});

					let infoWinCont = 'Total number of lost messages in <br/>sequence: ' + numOfLostMessages + '<br/> The next successful FCntUp: ' + FCntUpNew;

					google.maps.event.addListener(marker, 'click', function() {
						infoWindow.setOptions({
							content: infoWinCont,
							maxWidth: 400
						});
						infoWindow.open(map, this); 
					});
					markers.push(marker);
					bounds.extend(pos);
				}

			}

			pos = posNew;
			FCntUp = FCntUpNew;

			let LrrSNR = parseFloat(point.DevEUI_uplink.LrrSNR);
			for (var i=0; i<5; i++) {
				if ( (LrrSNR - adrMargin) < adrTresholds[i] ) break;
			}
		
			//console.log(i);

			let marker = new google.maps.Marker({
				position: pos, 
				//label: point.DevEUI_uplink.FCntUp,
				title: FCntUp+' | SF'+(12-i)+' | '+DevEUI,
				optimized: false,
				zIndex: 5,
				icon: pinSymbol(i)
			});

			google.maps.event.addListener(marker, 'click', 
				function() { 
					openInfoWindow(this, point.DevEUI_uplink);
					//openInfoWindow(this, point._id);
				}
			);

			markers.push(marker);
		
			bounds.extend(pos);
			
		}
		
	});

	if (done) {
		done();
	}
	else {
		show_markers();
		if ($('#autozoom').prop('checked')) {
			fit_map_to_all_markers();
		}
	}

}

function openInfoWindow(mkr, infoWinContent) {
	infoWindow.setOptions({
		content: '<pre>'+JSON.stringify(infoWinContent, null, 2)+'</pre>',
		maxWidth: 400
	});
	infoWindow.open(map, mkr); 
}

function show_markers() {
	hide_markers();			

	let size = parseInt($('#grid_size').val(), 10);
	size = size === -1 ? null : size;

	if (size) {			
		//let size = parseInt($("#grid_size").val(), 10);
		//size = size === -1 ? null : size;
		markerCluster = new MarkerClusterer(map, markers, {
			imagePath: 'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m',
			gridSize: size
		});
	}
	else {
		markers.forEach(function(marker, i){
			marker.setMap(map);
		});
	}

	$('body').removeClass('waiting');

}

function hide_markers() {
	markers.forEach(function(marker, i){
		marker.setMap(null);
	});
	if (markerCluster) {
		markerCluster.clearMarkers();
	}
}

function add_random_noice_to_markers() {

	markers.forEach(function(marker, i){
		marker.setPosition({
			lat: marker.getPosition().lat()+(Math.random()-0.5)/100000,
			lng: marker.getPosition().lng()+(Math.random()-0.5)/100000
		});

	});

}

function remove_markers() {
	hide_markers();
	points = {};
	markers = [];
	if (markerCluster) {
		markerCluster=null;
	}
}

function fit_map_to_all_markers() {
	if ( markers.length == 0 ) return;
	map.fitBounds(bounds);

	let cnt = map.getCenter();
    cnt.e+=0.000001;
    map.panTo(cnt);
    cnt.e-=0.000001;
    map.panTo(cnt);

}

function get_bounds() { 
	let currentBounds = map.getBounds();
	$("#startLat").val(currentBounds.getSouthWest().lat()); 
	$("#endLat").val(currentBounds.getNorthEast().lat()); 
	$("#startLng").val(currentBounds.getSouthWest().lng()); 
	$("#endLng").val(currentBounds.getNorthEast().lng());

	$('#startLat, #endLat, #startLng, #endLng').parent().find('i').show();

}

function clear_bounds() {
	$("#startLat").val(""); 
	$("#endLat").val(""); 
	$("#startLng").val(""); 
	$("#endLng").val("");
	$('#startLat, #endLat, #startLng, #endLng').parent().find('i').hide();
}

function zoom_to_bounds() {

/*
	let sw = new google.maps.LatLng({lat: filter.startLat, lng: filter.startLng});
	let ne = new google.maps.LatLng({lat: filter.endLat, lng: filter.endLng});
	let queryAreaBounds = new google.maps.LatLngBounds();
	queryAreaBounds.extend(sw);
	queryAreaBounds.extend(ne);
	map.fitBounds(queryAreaBounds);
*/
	map.fitBounds({
		east: filter.endLng,
		north: filter.endLat,
		west: filter.startLng,
		south: filter.startLat
	});
	map.setZoom(map.getZoom()+1);
}

function set_query_area_lines() {

		let areaBorders = [
		  {lat: filter.startLat, lng: filter.startLng},
		  {lat: filter.startLat, lng: filter.endLng},
		  {lat: filter.endLat, lng: filter.endLng},
		  {lat: filter.endLat, lng: filter.startLng},
		  {lat: filter.startLat, lng: filter.startLng}
		];

		if (queryAreaLines) queryAreaLines.setMap(null);

		queryAreaLines = new google.maps.Polygon({
			paths: [worldBorders, areaBorders],
			geodesic: true,
			strokeColor: '#000000',
			strokeOpacity: 0.8,
			strokeWeight: 2,
			fillColor: '#000000',
			fillOpacity: 0.35
		});

		queryAreaLines.setMap(map);

}

function clear_query_area_lines() {
	if (queryAreaLines) queryAreaLines.setMap(null);
	queryAreaLines = null;
}

function clear_times() {
  $("#startTime").val(""); 
  $("#endTime").val(""); 
}

function set_center_to_current_pos() {
	map.setCenter(currentPosition);
}





function first_page() {
	$("#pageNumber").val( 1 );
	get_points();
}

function next_page() {
	let n = parseInt($("#pageNumber").val());
	if ((n >= numberOfPages)&&(n!=0)) {
		n = numberOfPages;
		return;
	}
	$("#pageNumber").val( n+1 );
	get_points();
}

function prev_page() {
	let n = parseInt($("#pageNumber").val());
	if (n<0) {
		n = 0;
		return;
	}
	if (n==1) {
		return;
	}
	$("#pageNumber").val( n-1 );
	get_points();
}

function last_page() {
	let n = parseInt($("#numberOfPages").val());
	if (n==0) return;
	$("#pageNumber").val( $("#numberOfPages").val() );
	get_points();
}

function toggle_div(div_container) {
	$('#'+div_container).toggle();
}

function create_map() {

	let mapCanvas = document.getElementById("map");
	let mapOptions = {

		zoom: 13,
		center: defaultPosition,
		zoomControl: true,
		zoomControlOptions: {
			position: google.maps.ControlPosition.RIGHT_BOTTOM
		},
		streetViewControl: false,
		streetViewControlOptions: {
			position: google.maps.ControlPosition.RIGHT_BOTTOM
		},
		mapTypeControl: true,
		mapTypeControlOptions: {
			style: google.maps.MapTypeControlStyle.HORIZONTAL_BAR,
			position: google.maps.ControlPosition.LEFT_BOTTOM
		},
		fullscreenControl: true,
		fullscreenControlOptions: {
			position: google.maps.ControlPosition.RIGHT_BOTTOM
		}

	};
	map = new google.maps.Map(mapCanvas, mapOptions);
	infoWindow = new google.maps.InfoWindow();

	$.ajax({
		type: "GET",
		url: config.apiURL + '/gateway',
		dataType: "json",
		headers: {"Authorization": 'Bearer ' + access_token},
		data: { pageNumber: 1, pageLength: 50 }
	})
	.done(function( response, textStatus, jqXHR ) {

			gateways = response.response.items;
			set_gw_markers();

	})
	.fail(function( jqXHR, textStatus, errorThrown ) {
		if (jqXHR.status = 403) {
			// alert(jqXHR.responseJSON.message);
			location.href = 'login.html';
		} else {
			console.log('An error occurred.');
			console.log(errorThrown);
		}
	})

}
window.create_map = create_map;

$(document).ready(function() {

	$('#firstPage').click(first_page);
	$('#prevPage').click(prev_page);
	$('#nextPage').click(next_page);
	$('#lastPage').click(last_page);
	$('#allMarkers').click(fit_map_to_all_markers);
	$('#queryArea').click(zoom_to_bounds);
	$('#currentLocation').click(set_center_to_current_pos);
	$('#shakeMarkers').click(add_random_noice_to_markers);
	$('#refreshPage').click(function(){ get_points() });
	$('#getBounds').click(get_bounds);


	$('.ui.radio.checkbox.adr').checkbox();

	$('.ui.checkbox.showLostMessages').checkbox({

		beforeChecked: function(){

			if ( filter.DevEUI || filter.DevAddr ) {
				return true;
			}
			else {
				$('.ui.modal.showLostAlert').modal('show');
				return false;
			}
		},
		onChange: function(){ 
			get_points();
		}

	});



	$('#pageNumber').change(function(){ 
		get_points();
	});
	$('#pageLength').change(function(){
		$('#pageNumber').val(1);
		get_points();
	});


	$("#logout").click(function(){
		/*
		localStorage.removeItem(localStoragePrefix + 'token');
		localStorage.removeItem(localStoragePrefix + 'userId');
		localStorage.removeItem(localStoragePrefix + 'scope');
		localStorage.removeItem(localStoragePrefix + 'customerId');
		localStorage.removeItem(localStoragePrefix + 'iat');
		localStorage.removeItem(localStoragePrefix + 'exp');
		*/
		deleteCookie('access_token_' + config.client_id);
		window.location.href="login.html";
	});


	if (navigator.geolocation) {
		navigator.geolocation.getCurrentPosition(function(position) {
			currentPosition = {
				lat: position.coords.latitude,
				lng: position.coords.longitude
			};
		});
	}

	$("#navigation_container").hide();
	$("#filter_container").show();

	$('.ui.accordion').accordion();

	//$('.ui.checkbox').checkbox();

	$('#DevEUI_dropdown').dropdown({

		apiSettings: {
			// url: '/nano-map-api/v100/api/device?q=1',
			url: config.apiURL + '/device',
			beforeXHR: function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + access_token);
				return xhr;
			},
			onResponse: function(response) {
				let r = {results: [{name:'', value:''}]};
				response.response.items.forEach(function(item) {
					r.results.push({name: item.DevEUI, value: item.DevEUI})
				});
				r.success = true;
				return r;
			}
		},

/*
		apiSettings: {
			responseAsync: function(settings, callback) {
				const query = settings.urlData.query;
				let response = {
					success: true,
					results: [
						{name: '', value: ''},
						{name: '0018B20000000C87', value: '0018B20000000C87'},
						{name: '0018B20000000DEF', value: '0018B20000000DEF'},
						{name: '0018B20000001892', value: '0018B20000001892'},
						{name: '5605000000B21800', value: '5605000000B21800'}
					]
				};
				//console.log(JSON.stringify(settings.urlData));
				callback(response);
			},
		},
*/
		filterRemoteData: true
	});

	$('#DevAddr_dropdown').dropdown({


		apiSettings: {
			// url: '/nano-map-api/v100/api/device?q=2',
			url: config.apiURL + '/device',
			beforeXHR: function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + access_token);
				return xhr;
			},
			onResponse: function(response) {
				let r = {results: [{name:'', value:''}]};
				response.response.items.forEach(function(item) {
					r.results.push({name: item.DevAddr, value: item.DevAddr})
				});
				r.success = true;
				return r;
			}
		},
/*
		apiSettings: {
			responseAsync: function(settings, callback) {
				let response = {
					success: true,
					results: [
						{name: '', value: ''},
						{name: '870C0000', value: '870C0000'},
						{name: '85423567', value: '85423567'},
						{name: 'AB094EF7', value: 'AB094EF7'},
						{name: '45B1C09F', value: '45B1C09F'}
					]
				};
				callback(response);
			},
		},
*/
		filterRemoteData: true
	});
	
	$('#ADRbit_dropdown').dropdown();

	$('#SpFact_dropdown').dropdown();

	$('#bestLrrid_dropdown, #Lrrid_dropdown').dropdown({
		apiSettings: {
			// url: '/nano-map-api/v100/api/gateway?q=1',
			url: config.apiURL + '/gateway',
			beforeXHR: function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + access_token);
				return xhr;
			},
			onResponse: function(response) {
				let r = {results: [{name:'', value:''}]};
				response.response.items.forEach(function(item) {
					r.results.push({name: item.gatewayId, value: item.gatewayId})
				});
				r.success = true;
				return r;
			}
		},
		filterRemoteData: true
	});


	$('#marker_size_dropdown').dropdown({
		onChange: function(value,text){
			set_markers();
		}
	});

	$('#grid_size_dropdown').dropdown({
		onChange: function(value,text){
			show_markers();
		}
	});


	$('.ui.calendar').calendar({
		type: "datetime",
		formatter: {
			datetime: function (date, settings) { 
				if (!date) return '';
				return date.toISOString();
			}
		},
		onChange: function (date, text, mode) {
			$(this).parent().find('i').show();
			$('#filterStatus').html('- Not Activated!');
		}
	});




	$('.ui.radio.checkbox.qaBounds.auto').first().checkbox({
		onChecked: function() {
			$('.ui.field.segment.manualQASettings').hide();
		}
	});

	$('.ui.radio.checkbox.qaBounds.manual').first().checkbox({
		onChecked: function() {
			$('.ui.field.segment.manualQASettings').show();
		}
	});






	$('.ui.icon.input.clearfield i').click(function(){
		$(this).hide();
		$(this).parent().find('input').val('');
		$('#filterStatus').html('- Not Activated!');
	})

	$(".ui.icon.input.clearfield input").change( function() {
		if($(this).val()=='') {
			$(this).parent().find('i').hide();
		}
		else {
			$(this).parent().find('i').show();
		}
	}); 

	$.fn.form.settings.rules.exactLength0 = function(value, param) {
		return ((value == '')||(value.length == param)) ? true : false;
	}

	$('#applyFilter').click(function(){
		$('#filter').form('validate form');
	});

	$('#filter').form({
		fields: {
			DevEUI: {
				identifier: 'DevEUI',
				optional: true,
				rules: [
					{
						type   : 'exactLength[16]',
						prompt : 'Invalid DevEUI format.'
					}
				]
			},
			DevAddr: {
				identifier: 'DevAddr',
				optional: true,
				rules: [
					{
						type   : 'exactLength[8]',
						prompt : 'Invalid DevAddr format.'
					}
				]
			},
			bestLrrid: {
				identifier: 'bestLrrid',
				optional: true,
				rules: [
					{
						type   : 'exactLength[8]',
						prompt : 'Invalid bestLrrid format.'
					}
				]
			},
			Lrrid: {
				identifier: 'Lrrid',
				optional: true,
				rules: [
					{
						type   : 'exactLength[8]',
						prompt : 'Invalid Lrrid format.'
					}
				]
			},
			startLat: {
				identifier: 'startLat',
				optional: true,
				rules: [
					{
						type   : 'number',
						prompt : 'startLat has to be a number.'
					}
				]
			},
			endLat: {
				identifier: 'endLat',
				optional: true,
				rules: [
					{
						type   : 'number',
						prompt : 'endLat has to be a number.'
					}
				]
			},
			startLng: {
				identifier: 'startLng',
				optional: true,
				rules: [
					{
						type   : 'number',
						prompt : 'startLng has to be a number.'
					}
				]
			},
			endLng: {
				identifier: 'endLng',
				optional: true,
				rules: [
					{
						type   : 'number',
						prompt : 'endLng has to be a number.'
					}
				]
			}
		},
		onSuccess: function(){ 

			set_filter(get_points);
			$("#pageNumber").val("1");
			$('#nav-filter-panel').accordion('close', 0);
			$('#nav-filter-panel').accordion('close', 1);
			$('#filterStatus').html('');

		},
		onFailure: function(){ 

			$('#nav-filter-panel').accordion('close', 0);
			$('#nav-filter-panel').accordion('open', 1);

		}

	});
	$('#filter input').change(function(){
		$('#filterStatus').html('- Not Activated!');
	});

});

